@@ -1,5 +1,6 @@
 import functools
-import xml.etree.ElementTree as ET
+from lxml import etree as ET
+from xml.dom import minidom
 
 from tableaudocumentapi.property_decorators import argument_is_one_of
 
@@ -12,6 +13,7 @@ _ATTRIBUTES = [
     'alias',        # Name of the field as displayed in Tableau if the default name isn't wanted
     'calculation',  # If this field is a calculated field, this will be the formula
     'description',  # If this field has a description, this will be the description (including formatting tags)
+    'hidden',       # If this field has been hidden
 ]
 
 _METADATA_ATTRIBUTES = [
@@ -47,7 +49,7 @@ class Field(object):
         if column_xml is not None:
             self._initialize_from_column_xml(column_xml)
             self._xml = column_xml
-            # This isn't currently never called because of the way we get the data from the xml,
+            # This isn't currently called because of the way we get the data from the xml,
             # but during the refactor, we might need it.  This is commented out as a reminder
             # if metadata_xml is not None:
             #     self.apply_metadata(metadata_xml)
@@ -68,13 +70,15 @@ class Field(object):
             self._apply_attribute(xmldata, field_name,
                                   lambda x: getattr(xmldata.find('.//{}'.format(metadata_name)), 'text', None),
                                   read_name=metadata_name)
+            print(metadata_name, field_name)
         self.apply_metadata(xmldata)
 
     @classmethod
-    def create_field_xml(cls, caption, datatype, role, field_type, name):
+    def create_field_xml(cls, caption, datatype, hidden, role, field_type, name):
         column = ET.Element('column')
         column.set('caption', caption)
         column.set('datatype', datatype)
+        column.set('hidden', hidden)
         column.set('role', role)
         column.set('type', field_type)
         column.set('name', name)
@@ -135,6 +139,27 @@ class Field(object):
         """ XML representation of the field. """
         return self._xml
 
+    def pretty_xml(self):
+        """Return a pretty-printed XML string for the Element.
+        """
+        rough_string = ET.tostring(self._xml, 'utf-8')
+        prepared_string = minidom.parseString(rough_string)
+        print_string = prepared_string.toprettyxml(indent="  ", newl="")
+        return print_string.lstrip('<?xml version="1.0" ?>')
+
+    def __str__(self):
+        """ String representation of the field (only includes usable attributes) """
+        # TODO: ideally this should just loop through the ATTRIBUTES so it doesn't need touching for new ones
+        output = "------ FIELD {} ({}/{}/{}): {}(type), {}(datatype), {}(role), {}(aggregation)".format(
+            self.name, self.caption, self.alias, self.id, self.type, self.datatype, self.role, self.default_aggregation)
+        return output
+
+    def detailed_str(self):
+        if self.calculation:
+            calc = "\ncalc: `{}`".format(self.calculation)
+        else:
+            calc = ""
+
     ########################################
     # Attribute getters and setters
     ########################################
@@ -194,6 +219,25 @@ class Field(object):
         self._datatype = datatype
         self._xml.set('datatype', datatype)
 
+    @property
+    def hidden(self):
+        """ If the column is Hidden ('true', 'false') """
+        return self._hidden
+
+    @hidden.setter
+    @argument_is_one_of('true', 'false')
+    def hidden(self, hidden):
+        """ Set the hidden property of a field
+
+            Args:
+                hidden:  New hidden. String.
+
+            Returns:
+                Nothing.
+        """
+        self._hidden = hidden
+        self._xml.set('hidden', hidden)
+
     @property
     def role(self):
         """ Dimension or Measure """
@@ -215,7 +259,7 @@ class Field(object):
 
     @property
     def type(self):
-        """ Dimension or Measure """
+        """ Type of field (quantitative, ordinal, nominal) """
         return self._type
 
     @type.setter
@@ -251,7 +295,7 @@ class Field(object):
         # determine whether there already is an aliases-tag
         aliases = self._xml.find('aliases')
         # and create it if there isn't
-        if not aliases:
+        if not aliases:  # ignore the FutureWarning, does not apply to our usage
             aliases = ET.Element('aliases')
             self._xml.append(aliases)
 
@@ -272,7 +316,7 @@ class Field(object):
         Returns:
             Key-value mappings of all registered aliases. Dict.
         """
-        aliases_tag = self._xml.find('aliases') or []
+        aliases_tag = self._xml.find('aliases') or []    # ignore the FutureWarning, does not apply to our usage
         return {a.get('key', 'None'): a.get('value', 'None') for a in list(aliases_tag)}
 
     ########################################
@@ -336,6 +380,7 @@ class Field(object):
 
     @property
     def worksheets(self):
+        """ Worksheets which uses field. """
         return list(self._worksheets)
 
     ######################################
